/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.crop.persistence.serialization;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.EnumSet;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.crop.model.Crop;
import nl.cloudfarming.client.crop.model.CropManager;
import nl.cloudfarming.client.model.*;
import org.netbeans.api.project.Project;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author Wytse Visser
 */
@ServiceProvider(service = CropManager.class)
public class CropManagerImpl extends AbstractEntityManager<Crop> implements CropManager {

    private static final Logger LOGGER = Logger.getLogger(CropManager.class.getName());
    private static final String CROPS_FOLDER = "Crops";

    @Override
    public Set<ItemIdType> getSupportedTypes() {
        return EnumSet.of(ItemIdType.CRP);
    }

    @Override
    public void save(Crop crop) {
        try (ObjectOutputStream oos = new ObjectOutputStream(getCropFile(crop).getOutputStream())) {
            oos.writeObject(crop);
            addToCache(crop);
        } catch (IOException e) {
            LOGGER.log(Level.WARNING, "Could not save crop to file. {0}", e.getMessage());
        }
    }

    private FileObject getCropFile(Crop crop) throws IOException {
        FileObject fileObject = getCropFile(crop.getItemIdNumber());
        if (fileObject == null) {
            fileObject = createCropFile(crop);
        }
        return fileObject;
    }

    private FileObject getCropFile(String itemIdNumber) throws IOException {
        return getCropsFolder().getFileObject(itemIdNumber, Crop.EXT);
    }

    private FileObject createCropFile(Crop crop) throws IOException {
        return getCropsFolder().createData(crop.getItemIdNumber(), Crop.EXT);
    }

    protected FileObject getCropsFolder() throws IOException {
        FileObject folder = FileUtil.getConfigRoot().getFileObject(CROPS_FOLDER);
        if (folder == null) {
            folder = createCropsFolder();
        }
        return folder;
    }

    private FileObject createCropsFolder() throws IOException {
        return FileUtil.getConfigRoot().createFolder(CROPS_FOLDER);
    }

    @Override
    protected Crop load(AgroURI uri, Project project) {
        return load(uri);
    }

    @Override
    protected Crop load(AgroURI uri) {
        Crop crop = null;
        try (ObjectInputStream ois = new ObjectInputStream(getCropFile(uri.getItemIdNumber()).getInputStream())) {
            crop = (Crop) ois.readObject();
            crop = getCachedInstanceFor(crop);
        } catch (IOException | ClassNotFoundException e) {
            LOGGER.log(Level.WARNING, "Could not load crop from file. {0}", e.getMessage());
        }

        return crop;
    }

    @Override
    public void remove(Crop entity) {
        //TODO implement remove
        throw new UnsupportedOperationException("Not supported yet.");
    }
}
